@@ -1,18 +1,20 @@
 # Stdlib
 import json
 import logging
+import threading
+import functools
 
 # Pypi: Pika
 import pika
 
 # Django
 from django.conf import settings
+from django.db import connection as db_connection
 
 # Project
 from .config import get_config
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
 
 job_methods = {}
 
@@ -45,28 +47,67 @@ class JobClient(object):
 
     def __del__(self):
         if hasattr(self, 'channel'):
-            self.channel.close()
+            self.connection.close()
 
 
 class JobServer(object):
     job_methods = {}
+    threads = []
 
-    def __init__(self, channel, methods):
+    def __init__(self, channel, methods, connection):
         self.channel = channel
         self.job_methods = methods
-
-    def __del__(self):
-        self.channel.close()
+        self.connection = connection
 
     def start(self):
         self.channel.start_consuming()
 
-    def callback(self, ch, method, properties, body):
-        queue_task = self.job_methods[method.routing_key]
+    @staticmethod
+    def ack_message(ch, delivery_tag):
+        """Note that `ch` must be the same pika channel instance via which
+        the message being ACKed was retrieved (AMQP protocol constraint).
+        """
+        if ch.is_open:
+            ch.basic_ack(delivery_tag=delivery_tag)
+        else:
+            # Channel is already closed, so we can't ACK this message;
+            # log and/or do something that makes sense for your app in this case.
+            pass
+
+    @staticmethod
+    def do_work(connection, ch, delivery_tag, body, func, **kwargs):
+        func(body, **kwargs)
+        db_connection.close()
+        cb = functools.partial(JobServer.ack_message, ch, delivery_tag)
+        connection.add_callback_threadsafe(cb)
+
+    def callback(self, channel, method, properties, body):
+        body = json.loads(body)
+
+        logger.info('Start task: %s' % method.routing_key)
+        queue_task = self.job_methods.get(method.routing_key)
+        if not queue_task:
+            logger.info('End task: %s' % method.routing_key)
+            return channel.basic_ack(delivery_tag=method.delivery_tag)
+
         func = queue_task['func']
         kwargs = queue_task['kwargs']
-        func(json.loads(body), **kwargs)
-        ch.basic_ack(delivery_tag=method.delivery_tag)
+
+        thread = threading.Thread(
+            target=self.do_work,
+            args=(
+                self.connection,
+                self.channel,
+                method.delivery_tag,
+                body,
+                func
+            ),
+            kwargs=kwargs
+        )
+        thread.start()
+        self.threads.append(thread)
+
+        logger.info('End task: %s' % method.routing_key)
 
     def job_register(self):
         for key in self.job_methods.keys():
@@ -111,5 +152,6 @@ def job_tasks(routing_key, exchange='default', exchange_type='direct',
         }
 
         job_methods[routing_key] = data
+        return func
 
     return wrapper
