@@ -29,6 +29,15 @@ def start():
     channel = connection.channel()
 
     channel.basic_qos(prefetch_count=1)
-    job_server = JobServer(channel, job_methods)
+    job_server = JobServer(channel, job_methods, connection)
     job_server.job_register()
-    job_server.start()
+
+    try:
+        job_server.start()
+    except KeyboardInterrupt:
+        channel.stop_consuming()
+
+    for thread in job_server.threads:
+        thread.join()
+
+    connection.close()
