@@ -30,7 +30,7 @@ def get_new_fields(model):
 
 
 def pick(keys, dict_data):
-    return {key: dict_data[key] for key in keys if dict_data.get(key)}
+    return {key: dict_data[key] for key in keys if key in dict_data}
 
 
 def get_producer_models():
@@ -95,15 +95,18 @@ def model_sync(data, model):
 
     for item in data:
         defaults = pick(fields, item)
-        is_deleted = defaults.get('is_deleted')
-
-        if is_deleted:
-            model.objects.filter(id=item['id']).delete()
+        is_deleted = item.get('is_deleted')
+        pk = item.get('id')
+        if not pk:
             continue
 
-        model.objects.update_or_create(id=item['id'], defaults=defaults)
+        if is_deleted:
+            model.objects.filter(id=pk).delete()
+            continue
+
+        model.objects.update_or_create(id=pk, defaults=defaults)
         if m2m_fields:
-            instance = model.objects.get(id=item['id'])
+            instance = model.objects.get(id=pk)
 
             for field in m2m_fields:
                 if item.get(field):
